//
//  BFMusicSearchController.swift
//  BFRecordScreenKit
//
//  Created by Harry on 2022/3/3.
//

import Foundation
import BFUIKit
import BFMediaKit
import BFCommonKit

class BFMusicSearchController: BFBaseViewController {
    
    var choseAction : ((PQVoiceModel?) -> Void)?
    
    var loadedTimeRangesObserver : NSKeyValueObservation?

    var chosedCell : BFMuicInfoCell?

    // 试听音乐
    let player:AVPlayer = {
        let p = AVPlayer(playerItem: nil)
        return p
    }()
    
    lazy var musicTb : UITableView = {
        let tb = UITableView(frame: .zero)
        tb.delegate = self
        tb.dataSource = self
        tb.separatorColor = UIColor.hexColor(hexadecimal: "#272727")
        tb.register(BFMusicInfoSearchCell.self, forCellReuseIdentifier: "BFMusicInfoSearchCell")
        tb.tableFooterView = UIView()
        tb.backgroundColor = .clear
        return tb
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        let closeBtn = UIButton()
        closeBtn.setImage(imageInRecordScreenKit(by: "search_close"), for: .normal)
        closeBtn.frame = CGRect(x: 18, y: statusBarHeight + 2, width: 40, height: 40)
        view.addSubview(closeBtn)
        
        let searBar = UISearchBar(frame: CGRect(x: 62, y: statusBarHeight + 4, width: cScreenWidth - 62 - 18, height: 36))
        searBar.barStyle = .black
        searBar.prompt = "歌名/歌手名"
        
        view.addSubview(searBar)
        
        view.addSubview(musicTb)
        musicTb.snp.makeConstraints { make in
            make.left.right.bottom.equalToSuperview()
            make.top.equalTo(searBar.snp.bottom).offset(4)
        }
        
        loadedTimeRangesObserver = player.observe(\AVPlayer.currentItem?.loadedTimeRanges, options: [.new, .initial]) { [weak self] (player, change) in
            
            DispatchQueue.main.async {[weak self] in
                guard let wself = self else { return }
                
                guard let ranges = change.newValue as? [CMTimeRange] else { return }
                if let cell = wself.chosedCell, let totalDur = player.currentItem?.duration, totalDur.isValid,  CMTimeCompare(ranges.first?.duration ?? .zero, totalDur) >= 0{
                    if (player.currentItem?.asset as? AVURLAsset)?.url.absoluteString ?? "b" == cell.data?.musicPath ?? "a" {
                        if cell.status == .loading{
                            cell.status = .playing
                        }
                    }
                }
//                BFLog(1, message: "开始播放音乐:\(ranges.first?.start.seconds ?? 0), dur:\( ranges.first?.duration.seconds ?? 0), tot:\(player.currentItem?.duration.seconds ?? 0)")
            }
        }
        
    }
}

extension BFMusicSearchController : UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 4
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "BFMusicInfoSearchCell")
        
        if let cell = cell as? BFMusicInfoSearchCell{
            cell.useCallback = {[weak self] in
                guard let wself = self else { return }
                wself.choseAction?(cell.data)
                wself.dismiss(animated: true, completion: nil)
            }
        }
        
        return cell!
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 64
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if let cell = tableView.cellForRow(at: indexPath) as? BFMuicInfoCell {
//
//            chosedMusic = cell.data
//            chosedMusic?.isSelected = true
            cell.changeSelected(true)
            
            chosedCell = cell
            
            if cell.status == .normal {
                if let urlStr = cell.data?.musicPath, let url = URL(string: urlStr){
                    BFLog(1, message: "歌曲地址: \(url)")
                    if url.absoluteString != (player.currentItem?.asset as? AVURLAsset)?.url.absoluteString ?? "b" {
                        player.replaceCurrentItem(with: AVPlayerItem(url: url))
                        cell.status = .loading
                    }else {
                        cell.status = .playing
                    }
                    player.play()
                }
            }else if cell.status == .pause{
                player.play()
            } else{
                player.pause()
                cell.status = .pause
            }
        }
    }
    
    func tableView(_ tableView: UITableView, didDeselectRowAt indexPath: IndexPath) {
        let cell = tableView.cellForRow(at: indexPath) as? BFMuicInfoCell
        cell?.status = .normal
        cell?.changeSelected(false)
        
        chosedCell = nil
        player.pause()
    }
}
